@@ -14,8 +14,6 @@
 ### along with this program; if not, write to the Free Software
 ### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from __future__ import generators
-
 import paths
 import gnomeglade
 import gobject
@@ -24,6 +22,8 @@ import gtk.keysyms
 import math
 import misc
 import os
+from gettext import gettext as _
+from gettext import ngettext
 import shutil
 import melddoc
 import tree
@@ -39,16 +39,6 @@ gdk = gtk.gdk
 #
 ################################################################################
 
-def uniq(l):
-    i = iter(l)
-    a = i.next()
-    yield a
-    while 1:
-        b = i.next()
-        if a != b:
-            yield b
-            a = b
-
 _cache = {}
 
 def _files_same(lof, regexes):
@@ -98,14 +88,8 @@ def _files_same(lof, regexes):
     _cache[ lof ] = misc.struct(sigs=sigs, result=result)
     return result
 
-def _not_none(l):
-    """Return list with Nones filtered out"""
-    return filter(lambda x: x!=None, l)
-
-join = os.path.join
-
 COL_EMBLEM = tree.COL_END + 1
-pixbuf_newer = gnomeglade.load_pixbuf( paths.share_dir("glade2/pixmaps/tree-file-newer.png"), 14)
+pixbuf_newer = gnomeglade.load_pixbuf(paths.icon_dir("tree-file-newer.png"), 14)
 TYPE_PIXBUF = type(pixbuf_newer)
 
 ################################################################################
@@ -147,46 +131,18 @@ class EmblemCellRenderer(gtk.GenericCellRenderer):
 
     def on_render(self, window, widget, background_area, cell_area, expose_area, flags):
         r = self.renderer
-        r.set_property("pixbuf", self.pixbuf)
+        r.props.pixbuf = self.pixbuf
         r.render(window, widget, background_area, cell_area, expose_area, flags)
-        r.set_property("pixbuf", self.emblem)
+        r.props.pixbuf = self.emblem
         r.render(window, widget, background_area, cell_area, expose_area, flags)
 
     def on_get_size(self, widget, cell_area):
         if not hasattr(self, "size"):
             r = self.renderer
-            r.set_property("pixbuf", self.pixbuf)
+            r.props.pixbuf = self.pixbuf
             self.size = r.get_size(widget, cell_area)
         return self.size
 
-if gobject.pygtk_version < (2,8,0):
-    gobject.type_register(EmblemCellRenderer)
-
-################################################################################
-#
-# DirDiffMenu
-#
-################################################################################
-class DirDiffMenu(gnomeglade.Component):
-    def __init__(self, app):
-        gladefile = paths.share_dir("glade2/dirdiff.glade")
-        gnomeglade.Component.__init__(self, gladefile, "popup")
-        self.parent = app
-    def popup_in_pane( self, pane ):
-        self.copy_left.set_sensitive( pane > 0 )
-        self.copy_right.set_sensitive( pane+1 < self.parent.num_panes )
-        self.widget.popup( None, None, None, 3, gtk.get_current_event_time() )
-    def on_popup_compare_activate(self, menuitem):
-        self.parent.launch_comparisons_on_selected()
-    def on_popup_copy_left_activate(self, menuitem):
-        self.parent.on_button_copy_left_clicked( None )
-    def on_popup_copy_right_activate(self, menuitem):
-        self.parent.on_button_copy_right_clicked( None )
-    def on_popup_delete_activate(self, menuitem):
-        self.parent.on_button_delete_clicked( None )
-    def on_popup_edit_activate(self, menuitem):
-        self.parent.on_button_edit_clicked( None )
-
 ################################################################################
 #
 # TypeFilter
@@ -211,11 +167,45 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def __init__(self, prefs, num_panes):
         melddoc.MeldDoc.__init__(self, prefs)
-        gnomeglade.Component.__init__(self, paths.share_dir("glade2/dirdiff.glade"), "dirdiff")
-        self.toolbar.set_style( self.prefs.get_toolbar_style() )
-        self._map_widgets_into_lists( ["treeview", "fileentry", "diffmap", "scrolledwindow", "linkmap"] )
-        self.popup_menu = DirDiffMenu(self)
+        gnomeglade.Component.__init__(self, paths.ui_dir("dirdiff.glade"), "dirdiff")
+
+        actions = (
+            ("DirCompare",   gtk.STOCK_DIALOG_INFO,  _("_Compare"), None, _("Compare selected"), self.on_button_diff_clicked),
+            ("DirCopyLeft",  gtk.STOCK_GO_BACK,      _("Left"),     None, _("Copy To Left"), self.on_button_copy_left_clicked),
+            ("DirCopyRight", gtk.STOCK_GO_FORWARD,   _("Right"),    None, _("Copy To Right"), self.on_button_copy_right_clicked),
+            ("DirDelete",    gtk.STOCK_DELETE,        None,         None, _("Delete selected"), self.on_button_delete_clicked),
+            ("Hide",         gtk.STOCK_NO,           _("Hide"),     None, _("Hide selected"), self.on_filter_hide_current_clicked),
+
+            ("DirOpen",      gtk.STOCK_OPEN,          None,         None, _("Open selected"), self.on_button_open_clicked),
+        )
+
+        toggleactions = (
+            ("IgnoreCase",   gtk.STOCK_ITALIC,  _("Case"),     None, _("Ignore case of entries"), self.on_button_ignore_case_toggled, False),
+            ("ShowSame",     gtk.STOCK_APPLY,   _("Same"),     None, _("Show identical"), self.on_filter_state_normal_toggled, True),
+            ("ShowNew",      gtk.STOCK_ADD,     _("New"),      None, _("Show new"), self.on_filter_state_new_toggled, True),
+            ("ShowModified", gtk.STOCK_REMOVE,  _("Modified"), None, _("Show modified"), self.on_filter_state_modified_toggled, True),
+
+            ("CustomFilterMenu", None, _("Filters"), None, _("Set active filters"), self.on_custom_filter_menu_toggled, False),
+        )
+        self.ui_file = paths.ui_dir("dirdiff-ui.xml")
+        self.actiongroup = gtk.ActionGroup('DirdiffToolbarActions')
+        self.actiongroup.set_translation_domain("meld")
+        self.actiongroup.add_actions(actions)
+        self.actiongroup.add_toggle_actions(toggleactions)
+        self.create_name_filters()
+        for button in ("DirCompare", "DirCopyLeft", "DirCopyRight",
+                       "DirDelete", "Hide", "IgnoreCase", "ShowSame",
+                       "ShowNew", "ShowModified", "CustomFilterMenu"):
+            self.actiongroup.get_action(button).props.is_important = True
+        self.map_widgets_into_lists( ["treeview", "fileentry", "diffmap", "scrolledwindow", "linkmap"] )
         self.set_num_panes(num_panes)
+        self.focus_in_events = []
+        self.focus_out_events = []
+        for treeview in self.treeview:
+            handler_id = treeview.connect("focus-in-event", self.on_treeview_focus_in_event)
+            self.focus_in_events.append(handler_id)
+            handler_id = treeview.connect("focus-out-event", self.on_treeview_focus_out_event)
+            self.focus_out_events.append(handler_id)
         self.on_treeview_focus_out_event(None, None)
         self.treeview_focussed = None
 
@@ -238,7 +228,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             tree.STATE_MODIFIED,
             tree.STATE_NEW,
         ]
-        self.create_name_filters()
         self.update_regexes()
 
     def update_regexes(self):
@@ -251,12 +240,43 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     misc.run_dialog(
                         text=_("Error converting pattern '%s' to regular expression") % r.value )
 
+    def _custom_popup_deactivated(self, popup):
+        self.filter_menu_button.set_active(False)
+
+    def on_custom_filter_menu_toggled(self, item):
+        if item.get_active():
+            self.custom_popup.connect("deactivate", self._custom_popup_deactivated)
+            self.custom_popup.popup(None, None, misc.position_menu_under_widget,
+                                    1, gtk.get_current_event_time(), self.filter_menu_button)
+
+    def on_container_switch_in_event(self, ui):
+        melddoc.MeldDoc.on_container_switch_in_event(self, ui)
+        ui.insert_action_group(self.filter_actiongroup, -1)
+        self.custom_merge_id = ui.new_merge_id()
+        for x in self.filter_ui:
+            ui.add_ui(self.custom_merge_id, *x)
+        self.popup_deactivate_id = self.popup_menu.connect("deactivate", self.on_popup_deactivate_event)
+        self.custom_popup = ui.get_widget("/CustomPopup")
+        self.filter_menu_button = ui.get_widget("/Toolbar/FilterActions/CustomFilterMenu")
+        label = misc.make_tool_button_widget(self.filter_menu_button.props.label)
+        self.filter_menu_button.set_label_widget(label)
+
+        if self.treeview_focussed:
+            self.scheduler.add_task(self.treeview_focussed.grab_focus)
+            self.scheduler.add_task(self.on_treeview_cursor_changed)
+
+    def on_container_switch_out_event(self, ui):
+        self.popup_menu.disconnect(self.popup_deactivate_id)
+        ui.remove_ui(self.custom_merge_id)
+        ui.remove_action_group(self.filter_actiongroup)
+        melddoc.MeldDoc.on_container_switch_out_event(self, ui)
+
     def create_name_filters(self):
         self.name_filters_available = []
         for f in [misc.ListItem(s) for s in self.prefs.filters.split("\n") ]:
             bits = f.value.split()
             if len(bits) > 1:
-                regex = "(%s)$" % ")|(".join( [misc.shell_to_regex(b)[:-1] for b in bits] )
+                regex = "(%s)$" % "|".join( [misc.shell_to_regex(b)[:-1] for b in bits] )
             elif len(bits):
                 regex = misc.shell_to_regex(bits[0])
             else: # an empty pattern would match anything, skip it
@@ -268,24 +288,22 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             else:
                 func = lambda x, r=cregex : r.match(x) == None
                 self.name_filters_available.append( TypeFilter(f.name, f.active, func) )
-        self.name_filters = []
-        tips = gtk.Tooltips()
+        self.name_filters = [f for f in self.name_filters_available if f.active]
+
+        actions = []
+        self.filter_ui = []
         for i,f in enumerate(self.name_filters_available):
-            icon = gtk.Image()
-            icon.set_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_LARGE_TOOLBAR)
-            toggle = gtk.ToggleToolButton()
-            toggle.set_property("label",f.label)
-            toggle.set_icon_widget(icon)
-            toggle.connect("toggled", lambda b,i=i : self._update_name_filter(b,i) )
-            toggle.set_active(f.active)
-            self.toolbar.insert(toggle, -1)
-            toggle.show_all()
-            toggle.set_tooltip(tips, _("Hide %s") % f.label )
+            name = "Hide%d" % i
+            callback = lambda b, i=i: self._update_name_filter(b, i)
+            actions.append((name, None, f.label, None, _("Hide %s") % f.label, callback, f.active))
+            self.filter_ui.append(["/CustomPopup" , name, name, gtk.UI_MANAGER_MENUITEM, False])
+            self.filter_ui.append(["/Menubar/ViewMenu/FileFilters" , name, name, gtk.UI_MANAGER_MENUITEM, False])
+
+        self.filter_actiongroup = gtk.ActionGroup("DirdiffFilterActions")
+        self.filter_actiongroup.add_toggle_actions(actions)
 
     def on_preference_changed(self, key, value):
-        if key == "toolbar_style":
-            self.toolbar.set_style( self.prefs.get_toolbar_style() )
-        elif key == "regexes":
+        if key == "regexes":
             self.update_regexes()
 
     def _do_to_others(self, master, objects, methodname, args):
@@ -332,7 +350,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._update_diffmaps()
 
     def on_fileentry_activate(self, entry):
-        locs = [ e.get_full_path(0) for e in self.fileentry[:self.num_panes] ]
+        locs = [e.get_full_path() for e in self.fileentry[:self.num_panes]]
         self.set_locations(locs)
 
     def set_locations(self, locations):
@@ -341,6 +359,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.model.clear()
         for pane, loc in enumerate(locations):
             self.fileentry[pane].set_filename(loc)
+            self.fileentry[pane].prepend_history(loc)
         child = self.model.add_entries(None, locations)
         self.treeview0.grab_focus()
         self._update_item_state(child)
@@ -360,7 +379,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.scheduler.add_task( self._search_recursively_iter( path ).next )
 
     def _search_recursively_iter(self, rootpath):
-        self.filter_hide_current.set_sensitive(False)
+        self.actiongroup.get_action("Hide").set_sensitive(False)
         yield _("[%s] Scanning %s") % (self.label_text, "")
         prefixlen = 1 + len( self.model.value_path( self.model.get_iter(rootpath), 0 ) )
         symlinks_followed = {} # only follow symlinks once
@@ -373,7 +392,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             roots = self.model.value_paths( it )
             yield _("[%s] Scanning %s") % (self.label_text, roots[0][prefixlen:])
             differences = [0]
-            if not self.button_ignore_case.get_active():
+            if not self.actiongroup.get_action("IgnoreCase").get_active():
                 class accum(object):
                     def __init__(self, parent, roots):
                         self.items = []
@@ -381,11 +400,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     def add(self, pane, items):
                         self.items.extend(items)
                     def get(self):
-                        self.items.sort()
-                        def repeat(s, n):
-                            for i in xrange(n):
-                                yield s
-                        return [ tuple(repeat(i,self.n)) for i in  uniq(self.items) ]
+                        return [(i,) * self.n for i in sorted(set(self.items))]
             else:
                 canonicalize = lambda x : x.lower()
                 class accum(object):
@@ -415,12 +430,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                                 % "\n".join( self.bad ), self.parent )
                         keys = self.items.keys()
                         keys.sort()
-                        def fixup(key, tuples): # replace None with a usable label
+                        def fixup(tuples): # replace None with a usable label
                             def first_nonempty(seq):
                                 for s in seq:
                                     if s: return s
-                            return tuple([ t or first_nonempty(tuples) or k for t in tuples ])
-                        return [ fixup(k, self.items[k]) for k in keys ]
+                            return tuple([t or first_nonempty(tuples) for t in tuples])
+                        return [fixup(self.items[k]) for k in keys]
             accumdirs = accum(self, roots)
             accumfiles = accum(self, roots)
             for pane, root in enumerate(roots):
@@ -436,17 +451,20 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                         files = []
                         dirs = []
                         for e in entries:
-                            s = os.lstat( join(root,e) )
+                            try: # Necessary for some broken symlink cases; see bgo#585895
+                                s = os.lstat(os.path.join(root, e))
+                            except OSError, err:
+                                print "Ignoring OS error: %s" % err
+                                continue
                             if stat.S_ISLNK(s.st_mode):
                                 if not self.prefs.ignore_symlinks:
                                     key = (s.st_dev, s.st_ino)
                                     if symlinks_followed.get( key, 0 ) == 0:
                                         symlinks_followed[key] = 1
                                         try:
-                                            s = os.stat( join(root,e) )
+                                            s = os.stat(os.path.join(root, e))
                                         except OSError, err:
                                             print "ignoring dangling symlink", e
-                                            pass
                                         else:
                                             if stat.S_ISREG(s.st_mode):
                                                 files.append(e)
@@ -465,7 +483,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             # then directories and files
             if len(alldirs) + len(allfiles) != 0:
                 def add_entry(names):
-                    child = self.model.add_entries( it, [join(r,n) for r,n in zip(roots, names) ] )
+                    child = self.model.add_entries(it, [os.path.join(r, n) for r, n in zip(roots, names)])
                     differences[0] |= self._update_item_state(child)
                     return child
                 map(lambda x : todo.append( self.model.get_path(add_entry(x))), alldirs )
@@ -473,7 +491,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             else: # directory is empty, add a placeholder
                 self.model.add_empty(it)
             if differences[0]:
-                 expanded[path] = False
+                expanded[path] = False
         for path in sorted(expanded.keys()):
             start = path[:]
             while len(start) and not expanded.get(start,False):
@@ -485,7 +503,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 expanded[cur] = True
                 self.treeview[0].expand_row( cur, 0)
         yield _("[%s] Done") % self.label_text
-        self.filter_hide_current.set_sensitive(True)
+        self.actiongroup.get_action("Hide").set_sensitive(True)
 
     def launch_comparison(self, it, pane, force=1):
         """Launch comparison at 'it'. 
@@ -526,7 +544,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                         dstdir = os.path.dirname( dst )
                         if not os.path.exists( dstdir ):
                             os.makedirs( dstdir )
-                        shutil.copy2( src, dstdir )
+                        misc.copy2( src, dstdir )
                         self.file_created( path, dst_pane)
                     elif os.path.isdir(src):
                         if os.path.exists(dst):
@@ -555,7 +573,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                         os.remove(name)
                         self.file_deleted( path, pane)
                     elif os.path.isdir(name):
-                        if misc.run_dialog(_("'%s' is a directory.\nRemove recusively?") % os.path.basename(name),
+                        if misc.run_dialog(_("'%s' is a directory.\nRemove recursively?") % os.path.basename(name),
                                 parent = self,
                                 buttonstype=gtk.BUTTONS_OK_CANCEL) == gtk.RESPONSE_OK:
                             shutil.rmtree(name)
@@ -592,11 +610,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             else:
                 self.emit("status-changed", "%s : %s" % (rwx(stat.st_mode), nice(time.time() - stat.st_mtime) ) )
 
-    def on_switch_event(self):
-        if self.treeview_focussed:
-            self.scheduler.add_task( self.treeview_focussed.grab_focus )
-            self.scheduler.add_task( self.on_treeview_cursor_changed )
-
     def on_treeview_key_press_event(self, view, event):
         pane = self.treeview.index(view)
         tree = None
@@ -619,27 +632,22 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         return event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right) #handled
 
     def on_treeview_row_activated(self, view, path, column):
-        it = self.model.get_iter(path)
-        files = []
-        for i in range(self.num_panes):
-            fname = self.model.value_path( it, i )
-            if os.path.exists(fname):
-                files.append(fname)
+        pane = self.treeview.index(view)
+        allrows = self.model.value_paths(self.model.get_iter(path))
+        # Click a file: compare; click a directory: expand; click a missing
+        # entry: check the next neighbouring entry
+        pane_ordering = ((0, 1, 2), (1, 2, 0), (2, 1, 0))
+        for p in pane_ordering[pane]:
+            if p < self.num_panes and os.path.exists(allrows[p]):
+                pane = p
+                break
+        if os.path.isfile(allrows[pane]):
+            self.emit("create-diff", [r for r in allrows if os.path.isfile(r)])
+        elif os.path.isdir(allrows[pane]):
+            if view.row_expanded(path):
+                view.collapse_row(path)
             else:
-                files.append(None)
-        if files.count(None) != self.num_panes:
-            # Its possible to have file 'foo' in one pane and dir 'foo' in another.
-            # We want to do the right thing depending on the one clicked.
-            clicked_pane = [ t.get_column(0) for t in self.treeview ].index(column)
-            while files[clicked_pane] == None: # clicked on missing entry?
-                clicked_pane = (clicked_pane+1) % self.num_panes
-            if os.path.isfile( files[clicked_pane] ):
-                self.emit("create-diff", filter(os.path.isfile, _not_none(files) ))
-            else:
-                if view.row_expanded(path):
-                    view.collapse_row(path)
-                else:
-                    view.expand_row(path,0)
+                view.expand_row(path, False)
 
     def on_treeview_row_expanded(self, view, it, path):
         self._do_to_others(view, self.treeview, "expand_row", (path,0) )
@@ -649,18 +657,24 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._do_to_others(view, self.treeview, "collapse_row", (path,) )
         self._update_diffmaps()
 
+    def on_popup_deactivate_event(self, popup):
+        for (treeview, inid, outid) in zip(self.treeview, self.focus_in_events, self.focus_out_events):
+            treeview.handler_unblock(inid)
+            treeview.handler_unblock(outid)
+
     def on_treeview_focus_in_event(self, tree, event):
         self.treeview_focussed = tree
         pane = self.treeview.index(tree)
-        if pane > 0:
-            self.button_copy_left.set_sensitive(1)
-        if pane+1 < self.num_panes:
-            self.button_copy_right.set_sensitive(1)
-        self.button_delete.set_sensitive(1)
+        self.actiongroup.get_action("DirCopyLeft").set_sensitive(pane > 0)
+        self.actiongroup.get_action("DirCopyRight").set_sensitive(pane+1 < self.num_panes)
+        self.actiongroup.get_action("DirDelete").set_sensitive(True)
+        tree.emit("cursor-changed")
+
     def on_treeview_focus_out_event(self, tree, event):
-        self.button_copy_left.set_sensitive(0)
-        self.button_copy_right.set_sensitive(0)
-        self.button_delete.set_sensitive(0)
+        self.actiongroup.get_action("DirCopyLeft").set_sensitive(False)
+        self.actiongroup.get_action("DirCopyRight").set_sensitive(False)
+        self.actiongroup.get_action("DirDelete").set_sensitive(False)
+
         #
         # Toolbar handlers
         #
@@ -674,12 +688,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.copy_selected(1)
     def on_button_delete_clicked(self, button):
         self.delete_selected()
-    def on_button_edit_clicked(self, button):
+    def on_button_open_clicked(self, button):
         pane = self._get_focused_pane()
         if pane != None:
             m = self.model
             files = [ m.value_path( m.get_iter(p), pane ) for p in self._get_selected_paths(pane) ]
-            self._edit_files( [f for f in files if os.path.isfile(f)] )
+            self._open_files(files)
 
     def on_button_ignore_case_toggled(self, button):
         self.refresh()
@@ -722,10 +736,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         #
     def _get_selected_paths(self, pane):
         assert pane != None
-        selected_paths = []
-        self.treeview[pane].get_selection().selected_foreach(
-            lambda store, path, it: selected_paths.append( path ) )
-        return selected_paths
+        return self.treeview[pane].get_selection().get_selected_rows()[1]
 
         #
         # Filtering
@@ -781,6 +792,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             all_same = 0
             all_present_same = _files_same( lof, self.regexes )
         different = 1
+        one_isdir = [None for i in range(self.model.ntree)]
         for j in range(self.model.ntree):
             if mod_times[j]:
                 isdir = os.path.isdir( files[j] )
@@ -797,10 +809,20 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.model.set_value(it,
                     self.model.column_index(COL_EMBLEM, j),
                     j == newest_index and pixbuf_newer or None)
-            else:
-                self.model.set_state(it, j,  tree.STATE_MISSING)
+                one_isdir[j] = isdir
+        for j in range(self.model.ntree):
+            if not mod_times[j]:
+                self.model.set_state(it, j, tree.STATE_MISSING, True in one_isdir)
         return different
 
+    def popup_in_pane(self, pane):
+        for (treeview, inid, outid) in zip(self.treeview, self.focus_in_events, self.focus_out_events):
+            treeview.handler_block(inid)
+            treeview.handler_block(outid)
+        self.actiongroup.get_action("DirCopyLeft").set_sensitive(pane > 0)
+        self.actiongroup.get_action("DirCopyRight").set_sensitive(pane+1 < self.num_panes)
+        self.popup_menu.popup(None, None, None, 3, gtk.get_current_event_time())
+
     def on_treeview_button_press_event(self, treeview, event):
         # unselect other panes
         for t in filter(lambda x:x!=treeview, self.treeview[:self.num_panes]):
@@ -815,7 +837,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 selected = self._get_selected_paths( self.treeview.index(treeview) )
                 if len(selected) <= 1 and event.state == 0:
                     treeview.set_cursor( path, col, 0)
-                self.popup_menu.popup_in_pane( self.treeview.index(treeview) )
+                self.popup_in_pane( self.treeview.index(treeview) )
             return event.state==0
         return 0
 
@@ -971,7 +993,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             for component in changed:
                 child = model.iter_children( it )
                 while child:
-                    leading, name = os.path.split( model.value_path(child, pane) )
+                    name = os.path.basename(model.value_path(child, pane))
                     if component == name : # found it
                         it = child
                         break
@@ -1011,5 +1033,3 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_reload_activate(self, *extra):
         self.on_fileentry_activate(None)
 
-if gobject.pygtk_version < (2,8,0):
-    gobject.type_register(DirDiff)
