@@ -1,4 +1,5 @@
 ### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
+### Copyright (C) 2009 Vincent Legoll <vincent.legoll@gmail.com>
 
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -15,29 +16,44 @@
 ### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """Module of commonly used helper classes and functions
-
 """
 
-from __future__ import generators
 import copy
 import os
+from gettext import gettext as _
 import select
-import popen2
 import errno
 import gobject
 import gtk
 import shutil
 import re
 import signal
+import subprocess
 
 whitespace_re = re.compile(r"\s")
+NULL = open(os.devnull, "w+b")
+
+def cmdout(cmd, text=None, **kwargs):
+    stdin = NULL
+    if text is not None:
+        stdin = subprocess.PIPE
+    new_kwargs = {
+                  'stdin': stdin,
+                  'stdout': subprocess.PIPE,
+                  'stderr': NULL,
+                  }
+    new_kwargs.update(kwargs)
+    p = subprocess.Popen(cmd, **new_kwargs)
+    out = p.communicate(text)[0]
+    status = p.wait()
+    return out, status
 
 def shelljoin( command ):
     def quote(s):
         return ((whitespace_re.search(s) == None) and s or ('"%s"' % s))
     return " ".join( [ quote(x) for x in command ] )
 
-def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK, extrabuttons=[]):
+def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK, extrabuttons=()):
     """Run a dialog with text 'text'.
        Extra buttons are passed as tuples of (button label, response id).
     """
@@ -47,10 +63,10 @@ def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=
         messagetype,
         buttonstype,
         '<span weight="bold" size="larger">%s</span>' % escaped)
-    if parent:
+    if parent and isinstance(parent, gtk.Window):
         d.set_transient_for(parent.widget.get_toplevel())
     for b,rid in extrabuttons:
-        d.add_button(b,rid)
+        d.add_button(b, rid)
     d.vbox.set_spacing(12)
     hbox = d.vbox.get_children()[0]
     hbox.set_spacing(12)
@@ -62,6 +78,61 @@ def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=
     d.destroy()
     return ret
 
+def open_uri(uri, timestamp=0):
+    # TODO: should be 2.14 when released
+    if gtk.pygtk_version >= (2, 13, 0):
+        gtk.show_uri(gtk.gdk.screen_get_default(), uri, timestamp)
+    else:
+        try:
+            import gnome
+            gnome.url_show(uri)
+        except ImportError:
+            pass
+
+# Taken from epiphany
+def position_menu_under_widget(menu, widget):
+    container = widget.get_ancestor(gtk.Container)
+
+    widget_width, widget_height = widget.size_request()
+    menu_width, menu_height = menu.size_request()
+
+    screen = menu.get_screen()
+    monitor_num = screen.get_monitor_at_window(widget.window)
+    if monitor_num < 0:
+        monitor_num = 0
+    monitor = screen.get_monitor_geometry(monitor_num)
+
+    x, y = widget.window.get_origin()
+    if widget.flags() & gtk.NO_WINDOW:
+        x += widget.allocation.x
+        y += widget.allocation.y
+
+    if container.get_direction() == gtk.TEXT_DIR_LTR:
+        x += widget.allocation.width - widget_width
+    else:
+        x += widget_width - menu_width
+
+    if (y + widget.allocation.height + menu_height) <= monitor.y + monitor.height:
+        y += widget.allocation.height
+    elif (y - menu_height) >= monitor.y:
+        y -= menu_height
+    elif monitor.y + monitor.height - (y + widget.allocation.height) > y:
+        y += widget.allocation.height
+    else:
+        y -= menu_height
+
+    return (x, y, False)
+
+def make_tool_button_widget(label):
+    """Make a GtkToolButton label-widget suggestive of a menu dropdown"""
+    arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_NONE)
+    label = gtk.Label(label)
+    hbox = gtk.HBox(spacing=3)
+    hbox.pack_end(arrow)
+    hbox.pack_end(label)
+    hbox.show_all()
+    return hbox
+
 class struct(object):
     """Similar to a dictionary except that members may be accessed as s.member.
 
@@ -74,20 +145,20 @@ class struct(object):
     def __repr__(self):
         r = ["<"]
         for i in self.__dict__.keys():
-            r.append("%s=%s" % (i, getattr(self,i)))
+            r.append("%s=%s" % (i, getattr(self, i)))
         r.append(">\n")
         return " ".join(r)
     def __cmp__(self, other):
         return cmp(self.__dict__, other.__dict__)
 
-def all_equal(list):
+def all_equal(alist):
     """Return true if all members of the list are equal to the first.
 
     An empty list is considered to have all elements equal.
     """
-    if len(list):
-        first = list[0]
-        for n in list[1:]:
+    if len(alist):
+        first = alist[0]
+        for n in alist[1:]:
             if n != first:
                 return 0
     return 1
@@ -96,13 +167,10 @@ def shorten_names(*names):
     """Remove redunant parts of a list of names (e.g. /tmp/foo{1,2} -> foo{1,2}
     """
     prefix = os.path.commonprefix( names )
-    try:
-        prefixslash = prefix.rindex("/") + 1
-    except ValueError:
-        prefixslash = 0
-
-    names = map( lambda x: x[prefixslash:], names) # remove common prefix
-    paths = map( lambda x: x.split("/"), names) # split on /
+    prefixslash = prefix.rfind("/") + 1
+    
+    names = map(lambda x: x[prefixslash:], names) # remove common prefix
+    paths = map(lambda x: x.split("/"), names) # split on /
 
     try:
         basenames = map(lambda x: x[-1], paths)
@@ -110,29 +178,16 @@ def shorten_names(*names):
         pass
     else:
         if all_equal(basenames):
-            def firstpart(list):
-                if len(list) > 1: return "[%s] " % list[0]
-                else: return ""
+            def firstpart(alist):
+                if len(alist) > 1:
+                    return "[%s] " % alist[0]
+                else:
+                    return ""
             roots = map(firstpart, paths)
             base = basenames[0].strip()
             return [ r+base for r in roots ]
     # no common path. empty names get changed to "[None]"
-    return map( lambda x: x or _("[None]"), basenames)
-
-def look(s, o):
-    """Return a list of attributes in 'o' which contain the string 's'
-    """
-    return filter(lambda x:x.find(s)!=-1, dir(o))
-
-def ilook(s, o):
-    """Return a list of attributes in 'o' which contain the string 's' ignoring case. 
-    """
-    return filter(lambda x:x.lower().find(s)!=-1, dir(o))
-
-def all(o):
-    """Return a list of all the attributes in 'o' along with their values
-    """
-    return "\n".join( ["%s\t%s" % (x,getattr(o,x)) for x in dir(o)] )
+    return map(lambda x: x or _("[None]"), basenames)
 
 def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
     """Read the output of a shell command iteratively.
@@ -143,20 +198,17 @@ def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
     If 'workdir' is specified the command is run from that directory.
     """
     class sentinel(object):
+        def __init__(self):
+            self.proc = None
         def __del__(self):
-            if self.pipe:
-                errorstream.write("killing '%s' with pid '%i'\n" % (command[0], self.pipe.pid))
-                os.kill(self.pipe.pid, signal.SIGTERM)
-                errorstream.write("killed (status was '%i')\n" % self.pipe.wait())
+            if self.proc:
+                errorstream.write("killing '%s'\n" % command[0])
+                self.proc.terminate()
+                errorstream.write("killed (status was '%i')\n" % self.proc.wait())
         def __call__(self):
-            if workdir:
-                savepwd = os.getcwd()
-                os.chdir( workdir )
-            self.pipe = popen2.Popen3(command, capturestderr=1)
-            self.pipe.tochild.close()
-            childout, childerr = self.pipe.fromchild, self.pipe.childerr
-            if workdir:
-                os.chdir( savepwd )
+            self.proc = subprocess.Popen(command, cwd=workdir, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            self.proc.stdin.close()
+            childout, childerr = self.proc.stdout, self.proc.stderr
             bits = []
             while len(bits) == 0 or bits[-1] != "":
                 state = select.select([childout, childerr], [], [childout, childerr], yield_interval)
@@ -164,7 +216,7 @@ def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
                     if len(state[2]) == 0:
                         yield None
                     else:
-                        raise "Error reading pipe"
+                        raise Exception("Error reading pipe")
                 if childout in state[0]:
                     try:
                         bits.append( childout.read(4096) ) # get buffer size
@@ -175,42 +227,26 @@ def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
                         errorstream.write( childerr.read(1) ) # how many chars?
                     except IOError:
                         break # ick need to fix
-            status = self.pipe.wait()
+            status = self.proc.wait()
             errorstream.write( childerr.read() )
-            self.pipe = None
+            self.proc = None
             if status:
-               errorstream.write("Exit code: %i\n" % status)
+                errorstream.write("Exit code: %i\n" % status)
             yield "".join(bits)
     return sentinel()()
 
 def write_pipe(command, text):
     """Write 'text' into a shell command.
     """
-    pipe = popen2.Popen3(command, capturestderr=1)
-    pipe.tochild.write(text)
-    pipe.tochild.close()
-    return pipe.wait()
-
-def safe_apply(object, method, args):
-    """Call 'object.method(args)' if 'object' has an attribute named 'method'.
-
-    If 'object' has no method 'method' this is a no-op.
-    """
-    try:
-        m = getattr(object,method)
-    except AttributeError:
-        pass
-    else:
-        # allow single arguments to be passed as is.
-        if type(args) != type(()):
-            args = (args,)
-        apply(m, args)
+    proc = subprocess.Popen(command, stdin=subprocess.PIPE)
+    proc.communicate(text)
+    return proc.wait()
 
 def clamp(val, lower, upper):
     """Clamp 'val' to the inclusive range [lower,upper].
     """
     assert lower <= upper
-    return min( max(val, lower), upper)
+    return min(max(val, lower), upper)
 
 def commonprefix(dirs):
     """Given a list of pathnames, returns the longest common leading component.
@@ -222,27 +258,25 @@ def commonprefix(dirs):
     prefix = n[0]
     for item in n:
         for i in range(len(prefix)):
-            if prefix[:i+1] <> item[:i+1]:
+            if prefix[:i+1] != item[:i+1]:
                 prefix = prefix[:i]
-                if i == 0: return ''
+                if i == 0:
+                    return ''
                 break
     return os.sep.join(prefix)
 
-def escape(s):
-    """Replace special characters by SGML entities.
+def copy2(src, dst):
+    """Like shutil.copy2 but ignores chmod errors.
+    See [Bug 568000] Copying to NTFS fails
     """
-    entities = ("&&amp;", "<&lt;", ">&gt;")
-    for e in entities:
-        s = s.replace(e[0], e[1:])
-    return s
-
-def unescape(s):
-    """Inverse of escape.
-    """
-    entities = (">&gt;", "<&lt;", "&&amp;")
-    for e in entities:
-        s = s.replace(e[1:], e[0])
-    return s
+    if os.path.isdir(dst):
+        dst = os.path.join(dst, os.path.basename(src))
+    shutil.copyfile(src, dst)
+    try:
+        shutil.copystat(src, dst)
+    except OSError, e:
+        if e.errno != errno.EPERM:
+            raise
 
 def copytree(src, dst, symlinks=1):
     try:
@@ -260,7 +294,12 @@ def copytree(src, dst, symlinks=1):
         elif os.path.isdir(srcname):
             copytree(srcname, dstname, symlinks)
         else:
-            shutil.copy2(srcname, dstname)
+            copy2(srcname, dstname)
+
+def shell_escape(glob_pat):
+    # TODO: handle all cases
+    assert not re.compile(r"[][*?]").findall(glob_pat)
+    return glob_pat.replace('{', '[{]').replace('}', '[}]')
 
 def shell_to_regex(pat):
     """Translate a shell PATTERN to a regular expression.
@@ -320,3 +359,56 @@ class ListItem(object):
         self.value = " ".join(a)
     def __str__(self):
         return "<%s %s %i %s>" % ( self.__class__, self.name, self.active, self.value )
+
+
+################################################################################
+#
+# optparse Options subclass
+#
+################################################################################
+import optparse
+
+def check_diff_files(option, opt, value):
+    if len(value) not in (1, 2, 3):
+        raise optparse.OptionValueError(
+            "option %s: invalid value: %r" % (opt, value))
+
+def diff_files_callback(option, opt_str, value, parser):
+    """Gather arguments after option in a list and append to option.dest."""
+    assert value is None
+    diff_files_args = []
+    rargs = parser.rargs
+    while rargs:
+        arg = rargs[0]
+
+        # Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
+        # etc.  Note that this also stops on "-3" or "-3.0", so if
+        # your option takes numeric values, you will need to handle
+        # this.
+        if ((arg[:2] == "--" and len(arg) > 2) or
+            (arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
+            break
+        else:
+            diff_files_args.append(arg)
+            del rargs[0]
+
+    value = getattr(parser.values, option.dest) or []
+    value.append(diff_files_args)
+    setattr(parser.values, option.dest, value)
+
+
+class MeldOption(optparse.Option):
+    """Custom Option which adds the 'diff_files' action."""
+    TYPES = optparse.Option.TYPES + ("diff_files",)
+    TYPE_CHECKER = copy.copy(optparse.Option.TYPE_CHECKER)
+    TYPE_CHECKER["diff_files"] = check_diff_files
+
+    ACTIONS = optparse.Option.ACTIONS + ("diff_files",)
+    TYPED_ACTIONS = optparse.Option.TYPED_ACTIONS + ("diff_files",)
+
+    def take_action(self, action, dest, opt, value, values, parser):
+        if action == "diff_files":
+            diff_files_callback(self, opt, value, parser)
+        else:
+            optparse.Option.take_action(
+                self, action, dest, opt, value, values, parser)
