@@ -14,13 +14,11 @@
 ### along with this program; if not, write to the Free Software
 ### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from __future__ import generators
-
 import tempfile
 import shutil
 import gtk
 import os
-import re
+from gettext import gettext as _
 
 import tree
 import misc
@@ -59,7 +57,7 @@ def _commonprefix(files):
 ################################################################################
 class CommitDialog(gnomeglade.Component):
     def __init__(self, parent):
-        gnomeglade.Component.__init__(self, paths.share_dir("glade2/vcview.glade"), "commitdialog")
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.glade"), "commitdialog")
         self.parent = parent
         self.widget.set_transient_for( parent.widget.get_toplevel() )
         selected = parent._get_selected_files()
@@ -69,7 +67,8 @@ class CommitDialog(gnomeglade.Component):
         self.widget.show_all()
 
     def run(self):
-        self.previousentry.list.select_item(0)
+        self.previousentry.child.set_editable(False)
+        self.previousentry.set_active(0)
         self.textview.grab_focus()
         buf = self.textview.get_buffer()
         buf.place_cursor( buf.get_start_iter() )
@@ -79,53 +78,19 @@ class CommitDialog(gnomeglade.Component):
         if response == gtk.RESPONSE_OK:
             self.parent._command_on_selected( self.parent.vc.commit_command(msg) )
         if len(msg.strip()):
-            self.previousentry.prepend_history(1, msg)
+            self.previousentry.prepend_text(msg)
         self.widget.destroy()
     def on_previousentry_activate(self, gentry):
         buf = self.textview.get_buffer()
-        buf.set_text( gentry.gtk_entry().get_text() )
+        buf.set_text( gentry.child.get_text() )
 
 COL_LOCATION, COL_STATUS, COL_REVISION, COL_TAG, COL_OPTIONS, COL_END = range(tree.COL_END, tree.COL_END+6)
 
 class VcTreeStore(tree.DiffTreeStore):
     def __init__(self):
-        types = [type("")] * COL_END
-        types[tree.COL_ICON] = type(tree.pixbuf_file)
-        gtk.TreeStore.__init__(self, *types)
-        self.ntree = 1
-        self._setup_default_styles()
+        tree.DiffTreeStore.__init__(self, 1, COL_END)
         self.textstyle[tree.STATE_MISSING] = '<span foreground="#000088" strikethrough="true" weight="bold">%s</span>'
 
-################################################################################
-#
-# DirDiffMenu
-#
-################################################################################
-class VcMenu(gnomeglade.Component):
-    def __init__(self, app, event):
-        gladefile = paths.share_dir("glade2/vcview.glade")
-        gnomeglade.Component.__init__(self, gladefile, "menu_popup")
-        self.parent = app
-        self.widget.popup( None, None, None, 3, event.time )
-    def on_diff_activate(self, menuitem):
-        self.parent.on_button_diff_clicked( menuitem )
-    def on_edit_activate(self, menuitem):
-        self.parent._edit_files( self.parent._get_selected_files() )
-    def on_update_activate(self, menuitem):
-        self.parent.on_button_update_clicked( menuitem )
-    def on_commit_activate(self, menuitem):
-        self.parent.on_button_commit_clicked( menuitem )
-    def on_add_activate(self, menuitem):
-        self.parent.on_button_add_clicked( menuitem )
-    def on_add_binary_activate(self, menuitem):
-        self.parent.on_button_add_binary_clicked( menuitem )
-    def on_remove_activate(self, menuitem):
-        self.parent.on_button_remove_clicked( menuitem )
-    def on_revert_activate(self, menuitem):
-        self.parent.on_button_revert_clicked( menuitem )
-    def on_remove_locally_activate(self, menuitem):
-        self.parent.on_button_delete_clicked( menuitem )
-
 ################################################################################
 # filters
 ################################################################################
@@ -140,11 +105,56 @@ entry_ignored  = lambda x: (x.state == tree.STATE_IGNORED) or x.isdir
 #
 ################################################################################
 class VcView(melddoc.MeldDoc, gnomeglade.Component):
+    # Map action names to VC commands and required arguments list
+    action_vc_cmds_map = {
+                         "VcCompare": ("diff_command", ()),
+                         "VcCommit": ("commit_command", ("",)),
+                         "VcUpdate": ("update_command", ()),
+                         "VcAdd": ("add_command", ()),
+                         "VcAddBinary": ("add_command", ()),
+                         "VcResolved": ("resolved_command", ()),
+                         "VcRemove": ("remove_command", ()),
+                         "VcRevert": ("revert_command", ()),
+                         }
 
     def __init__(self, prefs):
         melddoc.MeldDoc.__init__(self, prefs)
-        gnomeglade.Component.__init__(self, paths.share_dir("glade2/vcview.glade"), "vcview")
-        self.toolbar.set_style( self.prefs.get_toolbar_style() )
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.glade"), "vcview")
+
+        actions = (
+            ("VcCompare",       gtk.STOCK_DIALOG_INFO,      _("_Compare"),      None, _("Compare selected"), self.on_button_diff_clicked),
+            ("VcOpen",          gtk.STOCK_OPEN,             None,               None, _("Open selected"), self.on_button_open_clicked),
+            ("VcCommit",        "vc-commit-24",             _("_Commit"),       None, _("Commit"), self.on_button_commit_clicked), # FIXME: popup used to use gtk.STOCK_GO_BACK
+            ("VcUpdate",        "vc-update-24",             _("_Update"),       None, _("Update"), self.on_button_update_clicked), # FIXME: popup used to use gtk.STOCK_GO_FORWARD
+            ("VcAdd",           "vc-add-24",                _("_Add"),          None, _("Add to VC"), self.on_button_add_clicked), # FIXME: popup used to use gtk.STOCK_ADD
+            ("VcAddBinary",     gtk.STOCK_ADD,              _("Add _Binary"),   None, _("Add binary to VC"), self.on_button_add_binary_clicked), # FIXME: stock is inconsistent with other VC actions
+            ("VcRemove",        "vc-remove-24",             _("_Remove"),       None, _("Remove from VC"), self.on_button_remove_clicked), # FIXME: popup used to use gtk.STOCK_REMOVE
+            ("VcResolved",      "vc-resolve-24",            _("_Resolved"),     None, _("Mark as resolved for VC"), self.on_button_resolved_clicked),
+            ("VcRevert",        gtk.STOCK_REVERT_TO_SAVED,  None,               None, _("Revert to original"), self.on_button_revert_clicked),
+            ("VcDeleteLocally", gtk.STOCK_DELETE,           None,               None, _("Delete locally"), self.on_button_delete_clicked), # FIXME: popup label was "_Remove locally"
+        )
+
+        toggleactions = (
+            ("VcFlatten",     gtk.STOCK_GOTO_BOTTOM, _("_Flatten"),  None, _("Flatten directories"), self.on_button_flatten_toggled, True),
+            ("VcShowModified","filter-modified-24",  _("_Modified"), None, _("Show modified"), self.on_button_filter_toggled, True),
+            ("VcShowNormal",  "filter-normal-24",    _("_Normal"),   None, _("Show normal"), self.on_button_filter_toggled, False),
+            ("VcShowNonVC",   "filter-nonvc-24",     _("Non _VC"),   None, _("Show unversioned files"), self.on_button_filter_toggled, False),
+            ("VcShowIgnored", "filter-ignored-24",   _("Ignored"),   None, _("Show ignored files"), self.on_button_filter_toggled, False),
+        )
+
+        self.ui_file = paths.ui_dir("vcview-ui.xml")
+        self.actiongroup = gtk.ActionGroup('VcviewActions')
+        self.actiongroup.set_translation_domain("meld")
+        self.actiongroup.add_actions(actions)
+        self.actiongroup.add_toggle_actions(toggleactions)
+        for action in ("VcCompare", "VcFlatten", "VcShowModified",
+                       "VcShowNormal", "VcShowNonVC", "VcShowIgnored"):
+            self.actiongroup.get_action(action).props.is_important = True
+        for action in ("VcCommit", "VcUpdate", "VcAdd", "VcRemove",
+                       "VcShowModified", "VcShowNormal", "VcShowNonVC",
+                       "VcShowIgnored", "VcResolved"):
+            button = self.actiongroup.get_action(action)
+            button.props.icon_name = button.props.stock_id
         self.tempdirs = []
         self.model = VcTreeStore()
         self.treeview.set_model(self.model)
@@ -174,29 +184,78 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         addCol(_("Options"), COL_OPTIONS)
 
         class ConsoleStream(object):
-            def __init__(this, textview):
-                this.textview = textview
+            def __init__(self, textview):
+                self.textview = textview
                 b = textview.get_buffer()
-                this.mark = b.create_mark("END", b.get_end_iter(), 0)
-            def write(this, s):
+                self.mark = b.create_mark("END", b.get_end_iter(), 0)
+            def write(self, s):
                 if s:
-                    b = this.textview.get_buffer()
+                    b = self.textview.get_buffer()
                     b.insert(b.get_end_iter(), s)
-                    this.textview.scroll_mark_onscreen( this.mark )
+                    self.textview.scroll_mark_onscreen( self.mark )
         self.consolestream = ConsoleStream(self.consoleview)
         self.location = None
-        self.treeview_column_location.set_visible( self.button_flatten.get_active() )
-        size = self.fileentry.size_request()[1]
-        self.button_jump.set_size_request(size, size)
-        self.button_jump.hide()
+        self.treeview_column_location.set_visible(self.actiongroup.get_action("VcFlatten").get_active())
+        self.fileentry.show() #TODO: remove once bug 97503 is fixed
         if not self.prefs.vc_console_visible:
             self.on_console_view_toggle(self.console_hide_box)
+        self.vc = None
+        # VC ComboBox
+        self.combobox_vcs = gtk.ComboBox()
+        self.combobox_vcs.lock = True
+        self.combobox_vcs.set_model(gtk.ListStore(str, object))
+        cell = gtk.CellRendererText()
+        self.combobox_vcs.pack_start(cell, False)
+        self.combobox_vcs.add_attribute(cell, 'text', 0)
+        self.combobox_vcs.lock = False
+        self.hbox2.pack_end(self.combobox_vcs, expand=False)
+        self.combobox_vcs.show()
+        self.combobox_vcs.connect("changed", self.on_vc_change)
+
+    def update_actions_sensitivity(self):
+        """Disable actions that use not implemented VC plugin methods
+        """
+        for action_name, (meth_name, args) in self.action_vc_cmds_map.items():
+            action = self.actiongroup.get_action(action_name)
+            try:
+                getattr(self.vc, meth_name)(*args)
+                action.props.sensitive = True
+            except NotImplementedError:
+                action.props.sensitive = False
+
+    def choose_vc(self, vcs):
+        """Display VC plugin(s) that can handle the location"""
+        self.combobox_vcs.lock = True
+        self.combobox_vcs.get_model().clear()
+        tooltip_texts = [_("Choose one Version Control"),
+                         _("Only one Version Control in this directory")]
+        default_active = 0
+        # Try to keep the same VC plugin active on refresh()
+        for idx, avc in enumerate(vcs):
+            if (self.vc is not None and
+                self.vc.__class__ == avc.__class__):
+                default_active = idx
+            self.combobox_vcs.get_model().append([avc.NAME, avc])
+        if gtk.pygtk_version >= (2, 12, 0):
+            self.combobox_vcs.set_tooltip_text(tooltip_texts[len(vcs) == 1])
+        self.combobox_vcs.set_sensitive(len(vcs) > 1)
+        self.combobox_vcs.lock = False
+        self.combobox_vcs.set_active(default_active)
+  
+    def on_vc_change(self, cb):
+        if not cb.lock:
+            self.vc = cb.get_model()[cb.get_active_iter()][1]
+            self._set_location(self.vc.root)
+            self.update_actions_sensitivity()
 
     def set_location(self, location):
+        self.choose_vc(vc.get_vcs(os.path.abspath(location or ".")))
+
+    def _set_location(self, location):
+        self.location = location
         self.model.clear()
-        self.location = location = os.path.abspath(location or ".")
-        self.fileentry.gtk_entry().set_text(location)
-        self.vc = vc.Vc(location)
+        self.fileentry.set_filename(location)
+        self.fileentry.prepend_history(location)
         it = self.model.add_entries( None, [location] )
         self.treeview.grab_focus()
         self.treeview.get_selection().select_iter(it)
@@ -216,18 +275,18 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         prefixlen = 1 + len( self.model.value_path( self.model.get_iter_root(), 0 ) )
         todo = [ (rootpath, rootname) ]
         filters = []
-        if self.button_modified.get_active():
+        if self.actiongroup.get_action("VcShowModified").get_active():
             filters.append( entry_modified )
-        if self.button_normal.get_active():
+        if self.actiongroup.get_action("VcShowNormal").get_active():
             filters.append( entry_normal )
-        if self.button_nonvc.get_active():
+        if self.actiongroup.get_action("VcShowNonVC").get_active():
             filters.append( entry_nonvc )
-        if self.button_ignored.get_active():
+        if self.actiongroup.get_action("VcShowIgnored").get_active():
             filters.append( entry_ignored )
         def showable(entry):
             for f in filters:
                 if f(entry): return 1
-        recursive = self.button_flatten.get_active()
+        recursive = self.actiongroup.get_action("VcFlatten").get_active()
         self.vc.cache_inventory(rootname)
         while len(todo):
             todo.sort() # depth first
@@ -261,12 +320,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.treeview.expand_row( (0,), 0)
         self.vc.uncache_inventory()
 
-    def on_preference_changed(self, key, value):
-        if key == "toolbar_style":
-            self.toolbar.set_style( self.prefs.get_toolbar_style() )
-
     def on_fileentry_activate(self, fileentry):
-        path = fileentry.get_full_path(0)
+        path = fileentry.get_full_path()
         self.set_location(path)
 
     def on_quit_event(self):
@@ -290,9 +345,9 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             path = self.model.value_path(it, 0)
             self.run_diff( [path] )
 
-    def run_diff_iter(self, paths, empty_patch_ok):
+    def run_diff_iter(self, path_list, empty_patch_ok):
         yield _("[%s] Fetching differences") % self.label_text
-        difffunc = self._command_iter(self.vc.diff_command(), paths, 0).next
+        difffunc = self._command_iter(self.vc.diff_command(), path_list, 0).next
         diff = None
         while type(diff) != type(()):
             diff = difffunc()
@@ -304,20 +359,21 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         elif empty_patch_ok:
             misc.run_dialog( _("No differences found."), parent=self, messagetype=gtk.MESSAGE_INFO)
         else:
-            for path in paths:
+            for path in path_list:
                 self.emit("create-diff", [path])
 
-    def run_diff(self, paths, empty_patch_ok=0):
-        self.scheduler.add_task( self.run_diff_iter(paths, empty_patch_ok).next, atfront=1 )
+    def run_diff(self, path_list, empty_patch_ok=0):
+        for path in path_list:
+            self.scheduler.add_task(self.run_diff_iter([path], empty_patch_ok).next, atfront=1)
 
     def on_button_press_event(self, text, event):
         if event.button==3:
-            VcMenu(self, event)
+            self.popup_menu.popup(None, None, None, 3, event.time)
             return len(self._get_selected_treepaths()) != 1
         return 0
 
     def on_button_flatten_toggled(self, button):
-        self.treeview_column_location.set_visible( self.button_flatten.get_active() )
+        self.treeview_column_location.set_visible(self.actiongroup.get_action("VcFlatten").get_active())
         self.refresh()
     def on_button_filter_toggled(self, button):
         self.refresh()
@@ -346,8 +402,9 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         msg = misc.shelljoin(command)
         yield "[%s] %s" % (self.label_text, msg.replace("\n", u"\u21b2") )
         def relpath(pbase, p):
-            assert p.startswith(pbase)
-            kill = len(pbase) and (len(pbase)+1) or 0
+            kill = 0
+            if len(pbase) and p.startswith(pbase):
+                kill = len(pbase) + 1
             return p[kill:] or "."
         if len(files) == 1 and os.path.isdir(files[0]):
             workdir = self.vc.get_working_directory(files[0])
@@ -381,28 +438,30 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         else:
             misc.run_dialog( _("Select some files first."), parent=self, messagetype=gtk.MESSAGE_INFO)
 
-    def on_button_update_clicked(self, object):
+    def on_button_update_clicked(self, obj):
         self._command_on_selected( self.vc.update_command() )
-    def on_button_commit_clicked(self, object):
+    def on_button_commit_clicked(self, obj):
         dialog = CommitDialog( self )
         dialog.run()
 
-    def on_button_add_clicked(self, object):
+    def on_button_add_clicked(self, obj):
         self._command_on_selected(self.vc.add_command() )
-    def on_button_add_binary_clicked(self, object):
+    def on_button_add_binary_clicked(self, obj):
         self._command_on_selected(self.vc.add_command(binary=1))
-    def on_button_remove_clicked(self, object):
+    def on_button_remove_clicked(self, obj):
         self._command_on_selected(self.vc.remove_command())
-    def on_button_revert_clicked(self, object):
+    def on_button_resolved_clicked(self, obj):
+        self._command_on_selected(self.vc.resolved_command())
+    def on_button_revert_clicked(self, obj):
         self._command_on_selected(self.vc.revert_command())
-    def on_button_delete_clicked(self, object):
+    def on_button_delete_clicked(self, obj):
         files = self._get_selected_files()
         for name in files:
             try:
                 if os.path.isfile(name):
                     os.remove(name)
                 elif os.path.isdir(name):
-                    if misc.run_dialog(_("'%s' is a directory.\nRemove recusively?") % os.path.basename(name),
+                    if misc.run_dialog(_("'%s' is a directory.\nRemove recursively?") % os.path.basename(name),
                             parent = self,
                             buttonstype=gtk.BUTTONS_OK_CANCEL) == gtk.RESPONSE_OK:
                         shutil.rmtree(name)
@@ -411,21 +470,20 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         workdir = _commonprefix(files)
         self.refresh_partial(workdir)
 
-    def on_button_diff_clicked(self, object):
+    def on_button_diff_clicked(self, obj):
         files = self._get_selected_files()
         if len(files):
             self.run_diff(files, empty_patch_ok=1)
 
-    def show_patch(self, prefix, patch):
-        if not patch: return
+    def on_button_open_clicked(self, obj):
+        self._open_files(self._get_selected_files())
 
+    def show_patch(self, prefix, patch):
         tmpdir = tempfile.mkdtemp("-meld")
         self.tempdirs.append(tmpdir)
 
-        regex = re.compile(self.vc.PATCH_INDEX_RE, re.M)
-        files = [f.strip() for f in regex.findall(patch)]
         diffs = []
-        for fname in files:
+        for fname in self.vc.get_patch_files(patch):
             destfile = os.path.join(tmpdir,fname)
             destdir = os.path.dirname( destfile )
 
@@ -439,15 +497,45 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             diffs.append( (destfile, pathtofile) )
 
         patchcmd = self.vc.patch_command( tmpdir )
-        misc.write_pipe(patchcmd, patch)
-        for d in diffs:
-            self.emit("create-diff", d)
+        if misc.write_pipe(patchcmd, patch) == 0:
+            for d in diffs:
+                self.emit("create-diff", d)
+        else:
+            import meldapp
+            msg = _("""
+                    Invoking 'patch' failed.
+                    
+                    Maybe you don't have 'GNU patch' installed,
+                    or you use an untested version of %s.
+                    
+                    Please send email bug report to:
+                    meld-list@gnome.org
+                    
+                    Containing the following information:
+                    
+                    - meld version: '%s'
+                    - source control software type: '%s'
+                    - source control software version: 'X.Y.Z'
+                    - the output of '%s somefile.txt'
+                    - patch command: '%s'
+                    (no need to actually run it, just provide
+                    the command line) 
+                    
+                    Replace 'X.Y.Z' by the actual version for the
+                    source control software you use.
+                    """) % (self.vc.NAME,
+                            meldapp.version,
+                            self.vc.NAME,
+                            " ".join(self.vc.diff_command()),
+                            " ".join(patchcmd))
+            msg = '\n'.join([line.strip() for line in msg.split('\n')])
+            misc.run_dialog(msg, parent=self)
 
     def refresh(self):
         self.set_location( self.model.value_path( self.model.get_iter_root(), 0 ) )
 
     def refresh_partial(self, where):
-        if not self.button_flatten.get_active():
+        if not self.actiongroup.get_action("VcFlatten").get_active():
             it = self.find_iter_by_name( where )
             if it:
                 newiter = self.model.insert_after( None, it)
@@ -458,52 +546,22 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         else: # XXX fixme
             self.refresh()
 
-    def on_button_jump_press_event(self, button, event):
-        class MyMenu(gtk.Menu):
-            def __init__(self, parent, where, showup=1):
-                gtk.Menu.__init__(self)
-                self.vcview = parent
-                self.map_id = self.connect("map", lambda item: self.on_map(item,where,showup) )
-            def add_item(self, name, submenu, showup):
-                item = gtk.MenuItem(name)
-                if submenu:
-                    item.set_submenu( MyMenu(self.vcview, submenu, showup ) )
-                self.append( item )
-            def on_map(self, item, where, showup):
-                if showup:
-                    self.add_item("..", os.path.dirname(where), 1 )
-                self.populate( where, self.listdir(where) )
-                self.show_all()
-                self.disconnect(self.map_id)
-                del self.map_id
-            def listdir(self, d):
-                try:
-                    return [p for p in os.listdir(d) if os.path.isdir( os.path.join(d,p))]
-                except OSError:
-                    return []
-            def populate(self, where, children):
-                for child in children:
-                    cc = self.listdir( os.path.join(where, child) )
-                    self.add_item( child, len(cc) and os.path.join(where,child), 0 )
-        menu = MyMenu( self, os.path.abspath(self.location) )
-        menu.popup(None, None, None, event.button, event.time)
-
     def _update_item_state(self, it, vcentry, location):
         e = vcentry
         self.model.set_state( it, 0, e.state, e.isdir )
-        def set(col, val):
-            self.model.set_value( it, self.model.column_index(col,0), val)
-        set( COL_LOCATION, location )
-        set( COL_STATUS, e.get_status())
-        set( COL_REVISION, e.rev)
-        set( COL_TAG, e.tag)
-        set( COL_OPTIONS, e.options)
+        def setcol(col, val):
+            self.model.set_value(it, self.model.column_index(col, 0), val)
+        setcol(COL_LOCATION, location)
+        setcol(COL_STATUS, e.get_status())
+        setcol(COL_REVISION, e.rev)
+        setcol(COL_TAG, e.tag)
+        setcol(COL_OPTIONS, e.options)
 
     def on_file_changed(self, filename):
         it = self.find_iter_by_name(filename)
         if it:
             path = self.model.value_path(it, 0)
-            dirs, files = self.vc.lookup_files( [], [ (os.path.basename(path), path)] )
+            files = self.vc.lookup_files([], [(os.path.basename(path), path)])[1]
             for e in files:
                 if e.path == path:
                     prefixlen = 1 + len( self.model.value_path( self.model.get_iter_root(), 0 ) )
